@@ -14,9 +14,9 @@ if [ -n "$SBOX_PRELOAD" ]; then
   # since the upgraded fakeroot in Scratchbox is apparently not all that
   # backwards compatible. Since I don't want to uninstall the Harmattan SDK
   # and reinstall the Fremantle SDK, I seem to need this hack.
-  aclocal-1.9 -I /targets/links/arch_tools/share/aclocal
+  aclocal-1.9 -I m4-repo -I /targets/links/arch_tools/share/aclocal
 else
-  aclocal-1.9
+  aclocal-1.9 -I m4-repo
 fi
 autoheader
 automake-1.9 -a -c
