@@ -1,14 +1,14 @@
 /**
  *  @File     stdlogicds.cpp
  *
- *  @Author   Lukas Zeller (luz@synthesis.ch)
+ *  @Author   Lukas Zeller (luz@plan44.ch)
  *
  *  @brief TStdLogicDS
  *    Standard database logic implementation, suitable for most (currently all)
  *    actual DS implementations, but takes as few assumptions about datastore
  *    so for vastly different sync patterns, this could be replaced by differnt locic
  *
- *    Copyright (c) 2001-2009 by Synthesis AG (www.synthesis.ch)
+ *    Copyright (c) 2001-2011 by Synthesis AG + plan44.ch
  *
  *  @Date 2005-09-15 : luz : created from custdbdatastore
  */
@@ -80,17 +80,21 @@ void TStdLogicDS::InternalResetDataStore(void)
   fInitializing=false;
   // no start init request yet
   fStartInit=false;
-  #if !defined(SYSYNC_CLIENT) || defined(CLIENT_USES_SERVER_DB)
-  // remove all items
-  TSyncItemPContainer::iterator pos;
-  for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
-    delete *pos;
+  if(HAS_SERVER_DB) {
+    #ifdef USES_SERVER_DB
+    // remove all items
+    TSyncItemPContainer::iterator pos;
+    for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
+      delete *pos;
+    }
+    fItems.clear(); // clear list
+    #endif
+  }
+  if (IS_SERVER) {
+    #ifdef SYSYNC_SERVER
+    fNumRefOnlyItems=0;
+    #endif
   }
-  fItems.clear(); // clear list
-  #endif
-  #ifndef SYSYNC_CLIENT
-  fNumRefOnlyItems=0;
-  #endif
 } // TStdLogicDS::InternalResetDataStore
 
 
@@ -148,23 +152,27 @@ localstatus TStdLogicDS::startDataWrite()
   localstatus sta = LOCERR_OK;
 
   if (!fWriteStarted) {
-    sta=implStartDataWrite();
+    sta = implStartDataWrite();
     fWriteStarted = sta==LOCERR_OK; // must be here to prevent recursion as startDataWrite might be called implicitly below
-    #if !defined(SYSYNC_CLIENT) || defined(CLIENT_USES_SERVER_DB)
-    // server-type DB needs post-processing to update map entries (client and server case)
-    TSyncItemPContainer::iterator pos;
-    if (sta==LOCERR_OK) {
-      // Now allow post-processing of all reported items
-      for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
-        localstatus sta2 = implReviewReadItem(**pos);
-        if (sta2!=LOCERR_OK) sta=sta2;
+    if (HAS_SERVER_DB) {
+      #ifdef USES_SERVER_DB
+      // server-type DB needs post-processing to update map entries (client and server case)
+      TSyncItemPContainer::iterator pos;
+      if (sta==LOCERR_OK) {
+        // Now allow post-processing of all reported items
+        for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
+          localstatus sta2 = implReviewReadItem(**pos);
+          if (sta2!=LOCERR_OK) sta = sta2;
+        }
       }
+      #endif
+    }
+    if (IS_CLIENT && HAS_SERVER_DB) {
+      #ifdef USES_SERVER_DB
+      /// @todo we don't need the items to remain that long at all - not even for CLIENT_USES_SERVER_DB case
+      fItems.clear(); // empty list
+      #endif
     }
-    #endif
-    #if defined(SYSYNC_CLIENT) && defined(CLIENT_USES_SERVER_DB)
-    /// @todo we don't need the items to remain that long at all - not even for CLIENT_USES_SERVER_DB case
-    fItems.clear(); // empty list
-    #endif // client using server DB
   }
   DEBUGPRINTFX(DBG_DATA,("startDataWrite called, status=%hd", sta));
   return sta;
@@ -207,12 +215,14 @@ bool TStdLogicDS::logicRetrieveItemByID(
 ///   or cannot be reached within the maximally allowed request processing time left.
 bool TStdLogicDS::isStarted(bool aWait)
 {
-  #ifndef SYSYNC_CLIENT
-  // only server has threaded datastores so far
-  if (aWait && fInitializing) {
-    localstatus sta = startDataAccessForServer();
-    if (sta!=LOCERR_OK) {
-      SYSYNC_THROW(TSyncException("startDataAccessForServer failed (when called from isStarted)", sta));
+  #ifdef SYSYNC_SERVER
+  if (IS_SERVER) {
+    // only server has threaded datastores so far
+    if (aWait && fInitializing) {
+      localstatus sta = startDataAccessForServer();
+      if (sta!=LOCERR_OK) {
+        SYSYNC_THROW(TSyncException("startDataAccessForServer failed (when called from isStarted)", sta));
+      }
     }
   }
   #endif
@@ -240,8 +250,43 @@ void TStdLogicDS::logicMarkItemForResend(cAppCharP aLocalID, cAppCharP aRemoteID
 } // TStdLogicDS::logicMarkItemForResend
 
 
+/// save status information required to possibly perform a resume (as passed to datastore with
+/// markOnlyUngeneratedForResume(), markItemForResume() and markItemForResend())
+/// (or, in case the session is really complete, make sure that no resume state is left)
+/// @note Must also save tempGUIDs (for server) and pending/unconfirmed maps (for client)
+localstatus TStdLogicDS::logicSaveResumeMarks(void)
+{
+  PDEBUGBLOCKFMTCOLL(("SaveResumeMarks","let implementation save resume info","datastore=%s",getName()));
+  localstatus sta = implSaveResumeMarks();
+  PDEBUGENDBLOCK("SaveResumeMarks");
+  return sta;
+} // TStdLogicDS::logicSaveResumeMarks
 
-#ifndef SYSYNC_CLIENT
+
+
+
+
+// - called for SyncML 1.1 if remote wants number of changes.
+//   Must return -1 if no NOC value can be returned
+sInt32 TStdLogicDS::getNumberOfChanges(void)
+{
+  if (IS_SERVER) {
+    #ifdef SYSYNC_SERVER
+    // for server, number of changes is the number of items in the item list
+    // minus those that are for reference only (in a slow sync resume)
+    return fItems.size()-fNumRefOnlyItems;
+    #endif
+  }
+  else {
+    // for client, derived class must provide it, or we'll return the default here (=no NOC)
+    // Note: for client-only builds, this methods does not exist in StdLogicDS and thus
+    //       inherited is always used
+    return inherited::getNumberOfChanges();
+  }
+} // TStdLogicDS::getNumberOfChanges
+
+
+#ifdef SYSYNC_SERVER
 
 // Server case
 // ===========
@@ -298,16 +343,21 @@ localstatus TStdLogicDS::performStartSync(void)
             // we need to post-fetch filter the item first
             bool passes=postFetchFiltering(myitemP);
             if (!passes) {
-              // item does not pass = does not belong to sync set per now
-              if (!fSlowSync && (sop==sop_wants_replace)) {
+              // item was changed and does not pass now -> might be fallen out of the sync set now.
+              // Only when the DB is capable of tracking items fallen out of the sync set (i.e. bring them up as adds
+              // later should they match the filter criteria again), we can implement removing based
+              // on filter criteria. Otherwise, these are simply ignored.
+              if (implTracksSyncopChanges() && !fSlowSync && (sop==sop_wants_replace)) {
                 // item already exists on remote but falls out of syncset now: delete
                 // NOTE: This works only if reviewReadItem() is correctly implemented
                 //       and checks for items that are deleted after being reported
-                //       something else to delete their local map entry
+                //       as replace to delete their local map entry (which makes
+                //       them add candidates again)
                 sop=sop_delete;
                 myitemP->cleardata(); // also get rid of unneeded data
               }
-              else sop=sop_none; // ignore all others (especially adds or slowsync replaces)
+              else
+                sop=sop_none; // ignore all others (especially adds or slowsync replaces)
             }
             else {
               // item passes = belongs to sync set
@@ -351,7 +401,7 @@ localstatus TStdLogicDS::performStartSync(void)
     // end reading
     sta=implEndDataRead();
     // show items
-    PDEBUGPRINTFX(DBG_HOT,("%s: number of local items involved in %ssync = %ld",getName(), fSlowSync ? "slow " : "",fItems.size()));
+    PDEBUGPRINTFX(DBG_HOT,("%s: number of local items involved in %ssync = %ld",getName(), fSlowSync ? "slow " : "",(long)fItems.size()));
     CONSOLEPRINTF(("  %ld local items are new/changed/deleted for this sync",fItems.size()));
     if (PDEBUGTEST(DBG_DATA+DBG_DETAILS)) {
       PDEBUGBLOCKFMTCOLL(("SyncSet","Items involved in Sync","datastore=%s",getName()));
@@ -394,7 +444,7 @@ localstatus TStdLogicDS::performStartSync(void)
 #ifdef MULTI_THREAD_DATASTORE
 
 // function executed by thread
-static uInt32 StartSyncThreadFunc(TThreadObject *aThreadObject, uInt32 aParam)
+static uInt32 StartSyncThreadFunc(TThreadObject *aThreadObject, uIntArch aParam)
 {
   // parameter passed is pointer to datastore
   TStdLogicDS *datastoreP = static_cast<TStdLogicDS *>((void *)aParam);
@@ -414,7 +464,7 @@ bool TStdLogicDS::threadedStartSync(void)
   // Note: ThreadMayChangeNow() has been posted already by startingThread()
   PDEBUGPRINTFX(DBG_HOT,("******* starting background thread for reading sync set..."));
   fStartSyncStatus.setStatusCode(200); // assume ok
-  if (!fStartSyncThread.launch(StartSyncThreadFunc,(uInt32)this)) { // pass datastoreP as param
+  if (!fStartSyncThread.launch(StartSyncThreadFunc,(uIntArch)this)) { // pass datastoreP as param
     // starting thread failed
     PDEBUGPRINTFX(DBG_ERROR,("******* Failed starting background thread for reading sync set"));
     return false;
@@ -440,7 +490,7 @@ bool TStdLogicDS::shouldExitStartSync(void)
   return fMultiThread && fStartSyncThread.terminationRequested();
 } // TStdLogicDS::shouldExitStartSync
 
-#else
+#else // MULTI_THREAD_DATASTORE
 
 // can be called to check if performStartSync() should be terminated
 bool TStdLogicDS::shouldExitStartSync(void)
@@ -449,7 +499,7 @@ bool TStdLogicDS::shouldExitStartSync(void)
   return false;
 } // TStdLogicDS::shouldExitStartSync
 
-#endif
+#endif // MULTI_THREAD_DATASTORE
 
 
 // called by dsBeforeStateChange to dssta_dataaccessstarted to make sure datastore is
@@ -465,7 +515,7 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
     // - start initialisation now
     fWriteStarted=false;
     // - read all records from DB right now if server data is used at all
-    DEBUGPRINTFX(DBG_DATA,("- number of items in list before StartDataRead = %ld",fItems.size()));
+    DEBUGPRINTFX(DBG_DATA,("- number of items in list before StartDataRead = %ld",(long)fItems.size()));
     // now we can initialize the conflict resolution mode for this session
     /// @todo move this to localengineds, at point where we get dssta_syncmodestable
     fSessionConflictStrategy=getConflictStrategy(fSlowSync,fFirstTimeSync);
@@ -478,6 +528,10 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
       &DBFuncTable, // context's function table
       this // datastore pointer needed for context
     );
+    // - datastoreinitscript might abort the sync with this datastore, check for that and exit if so
+    if (isAborted()) {
+      return getAbortStatusCode();
+    }
     #endif
     // - init post fetch filtering, sets fFilteringNeededForAll and fFilteringNeeded correctly
     initPostFetchFiltering();
@@ -485,7 +539,7 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
     fInitializing=true; // we enter the initialisation phase now
     fStartInit=true; // and we want to start the init
   }
-  // try starting init now (eventually repeats until it can be done)
+  // try starting init now (possibly repeats until it can be done)
   if (fStartInit) {
     PDEBUGPRINTFX(DBG_DATA,( "MultiThread %sabled", fMultiThread ? "en":"dis" ));
     #ifdef MULTI_THREAD_DATASTORE // combined define and flag
@@ -529,7 +583,7 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
     if (sta==LOCERR_OK) {
       // quick test: if number of items is > than allowed maxid of remote datatstore,
       // sync is unlikely to succeed
-      if (getRemoteDatastore()->getMaxID()<fItems.size()) {
+      if (getRemoteDatastore()->getMaxID()<(long)fItems.size()) {
         // this will not work, warn (but no longer abort session, as Siemens S55 guys don't like that)
         CONSOLEPRINTF((
           "Warning: Synchronisation involves more items (%ld) than client can possibly manage (%ld",
@@ -538,8 +592,8 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
         ));
         PDEBUGPRINTFX(DBG_ERROR,(
           "Warning: Synchronisation involves more items (%ld) than client can possibly manage (%ld)",
-          (sInt32)fItems.size(),
-          (sInt32)getRemoteDatastore()->getMaxID()
+          (long)fItems.size(),
+          (long)getRemoteDatastore()->getMaxID()
         ));
       }
     }
@@ -557,7 +611,7 @@ localstatus TStdLogicDS::startDataAccessForServer(void)
 // called to check if conflicting replace or delete command from server exists
 TSyncItem *TStdLogicDS::getConflictingItemByRemoteID(TSyncItem *syncitemP)
 {
-  // search for conflicting item by LUID
+  // search for conflicting item by remoteID
   TSyncItemPContainer::iterator pos;
   for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
     if (strcmp((*pos)->getRemoteID(),syncitemP->getRemoteID())==0) {
@@ -576,6 +630,31 @@ TSyncItem *TStdLogicDS::getConflictingItemByRemoteID(TSyncItem *syncitemP)
 } // TStdLogicDS::getConflictingItemByRemoteID
 
 
+
+// called to check if conflicting item (with same localID) already exists in the list of items
+// to be sent to the server
+TSyncItem *TStdLogicDS::getConflictingItemByLocalID(TSyncItem *syncitemP)
+{
+  // search for conflicting item by localID
+  TSyncItemPContainer::iterator pos;
+  for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
+    if (strcmp((*pos)->getLocalID(),syncitemP->getLocalID())==0) {
+      // same LUID exists in data from server
+      PDEBUGPRINTFX(DBG_DATA+DBG_CONFLICT,(
+        "TStdLogicDS::getConflictingItemByLocalID, found RemoteID='%s', LocalID='%s', syncop=%s",
+        syncitemP->getRemoteID(),
+        syncitemP->getLocalID(),
+        SyncOpNames[syncitemP->getSyncOp()]
+      ));
+      return (*pos); // return pointer to item in question
+    }
+  }
+  PDEBUGPRINTFX(DBG_DATA+DBG_CONFLICT,("TStdLogicDS::getConflictingItemByLocalID, no conflicting item"));
+  return NULL;
+} // TStdLogicDS::getConflictingItemByLocalID
+
+
+
 // called to check if content-matching item from server exists for slow sync
 TSyncItem *TStdLogicDS::getMatchingItem(TSyncItem *syncitemP, TEqualityMode aEqMode)
 {
@@ -630,7 +709,7 @@ void TStdLogicDS::dontSendItemAsServer(TSyncItem *syncitemP)
   for (pos=fItems.begin(); pos!=fItems.end(); ++pos) {
     if (*pos == syncitemP) {
       // it is in our list
-      PDEBUGPRINTFX(DBG_DATA+DBG_HOT,("Item with localID='%s' will NOT be sent to client (usually due to slowsync match)",syncitemP->getLocalID()));
+      PDEBUGPRINTFX(DBG_DATA+DBG_HOT,("Item with localID='%s' will NOT be sent to client (slowsync match / duplicate prevention)",syncitemP->getLocalID()));
       delete *pos; // delete item itself
       fItems.erase(pos); // remove from list
       break;
@@ -640,7 +719,7 @@ void TStdLogicDS::dontSendItemAsServer(TSyncItem *syncitemP)
 
 
 // - called when a item in the sync set changes its localID (due to local DB internals)
-//   Datastore must make sure that eventually cached items get updated
+//   Datastore must make sure that possibly cached items get updated
 // - NOTE: derivates must take care of updating map entries as well!
 void TStdLogicDS::dsLocalIdHasChanged(const char *aOldID, const char *aNewID)
 {
@@ -686,45 +765,13 @@ bool TStdLogicDS::MapFinishAsServer(
 } // TStdLogicDS::MapFinishAsServer
 
 
-// - called for SyncML 1.1 if remote wants number of changes.
-//   Must return -1 if no NOC value can be returned
-//   NOTE: we implement it here only for server, as it is not really needed
-//   for clients normally - if it is needed, client's agent must provide
-//   it as CustDBDatastore has no own list it can use to count in client case.
-sInt32 TStdLogicDS::getNumberOfChanges(void)
-{
-  // for server, number of changes is the number of items in the item list
-  // minus those that are for reference only (in a slow sync resume)
-  return fItems.size()-fNumRefOnlyItems;
-} // TStdLogicDS::getNumberOfChanges
-
-
-/// @brief called to have all non-yet-generated sync commands as "to-be-resumed"
-void TStdLogicDS::logicMarkOnlyUngeneratedForResume(void)
-{
-  // we do not maintain the map/bookmark list at this level, so
-  // derived class (ODBC, BinFile etc.) must make sure that their list is
-  // clean (no marks from previous sessions) before calling this inherited version
-  implMarkOnlyUngeneratedForResume();
-  // Now add those that we have already received from the implementation
-  TSyncItemPContainer::iterator pos;
-  for (pos = fItems.begin(); pos != fItems.end(); ++pos) {
-    // let datastore mark these unprocessed
-    TSyncItem *syncitemP = (*pos);
-    // mark it for resume by ID
-    logicMarkItemForResume(syncitemP->getLocalID(),syncitemP->getRemoteID(),true); // these are unsent
-  }
-} // TStdLogicDS::logicMarkOnlyUngeneratedForResume
-
-
-
 // - called to let server generate sync commands for client
 //   Returns true if now finished (or aborted) for this datastore
 //   also sets fState to dss_syncdone when finished
 bool TStdLogicDS::logicGenerateSyncCommandsAsServer(
   TSmlCommandPContainer &aNextMessageCommands,
   TSmlCommand * &aInterruptedCommandP,
-  const char *aLocalIDPrefix
+  cAppCharP aLocalIDPrefix
 )
 {
   bool alldone=false;
@@ -740,14 +787,14 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsServer(
   if (fMaxItemCount> 0) {
     PDEBUGPRINTFX(DBG_DATA+DBG_HOT,(
       "Info: Max number of items to be sent in this session is limited to %ld (already sent by now=%ld)",
-      fMaxItemCount,
-      fItemsSent
+      (long)fMaxItemCount,
+      (long)fItemsSent
     ));
   }
   #endif
   while (
     !isAborted() && // not aborted
-    (getDSConfig()->fMaxItemsPerMessage==0 || itemcount<getDSConfig()->fMaxItemsPerMessage==0) && // max item count per message not reached or not active
+    (getDSConfig()->fMaxItemsPerMessage==0 || itemcount<getDSConfig()->fMaxItemsPerMessage) && // max item count per message not reached or not active
     !fSessionP->outgoingMessageFull() && // message not full
     aNextMessageCommands.size()==0 // no commands already queued for next message
   ) {
@@ -792,7 +839,7 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsServer(
           ));
           ignoreitem=true;
         }
-        // - check if item passes eventual TAF
+        // - check if item passes possible TAF
         /// %%% (do not filter replaces, as these would not get reported again in the next session)
         /// @todo: the above is no longer true as we can now have them re-sent in next session,
         ///        so this must be changed later!!!
@@ -819,16 +866,8 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsServer(
       // test next
       continue;
     }
-    // add prefixes to ID
-    if (syncitemP->hasLocalID()) {
-      // make sure GUID (plus prefixes) is not exceeding allowed size
-      adjustLocalIDforSize(syncitemP->fLocalID,getRemoteDatastore()->getMaxGUIDSize(),aLocalIDPrefix ? strlen(aLocalIDPrefix) : 0);
-      // add local ID prefix, if any
-      if (aLocalIDPrefix && *aLocalIDPrefix)
-        syncitemP->fLocalID.insert(0,aLocalIDPrefix);
-    }
     // create sync op command (may return NULL in case command cannot be created, e.g. for MaxObjSize limitations)
-    TSyncOpCommand *syncopcmdP = newSyncOpCommand(syncitemP,itemtypeP);
+    TSyncOpCommand *syncopcmdP = newSyncOpCommand(syncitemP,itemtypeP,aLocalIDPrefix);
     // erase item from list
     delete syncitemP;
     pos = fItems.erase(pos);
@@ -839,7 +878,7 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsServer(
     //   would be sent before the third..nth chunk of the previous command).
     TSmlCommand *cmdP = syncopcmdP;
     syncopcmdP=NULL;
-    // eventually, we have a NULL command here (e.g. in case it could not be generated due to MaxObjSize restrictions)
+    // possibly, we have a NULL command here (e.g. in case it could not be generated due to MaxObjSize restrictions)
     if (cmdP) {
       if (!fSessionP->issuePtr(cmdP,aNextMessageCommands,aInterruptedCommandP)) {
         alldone=false; // issue failed (no room in message), not finished so far
@@ -849,7 +888,7 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsServer(
       fItemsSent++; // overall counter for statistics
       itemcount++; // per message counter
       // send event (but no check for abort)
-      OBJ_PROGRESS_EVENT(fSessionP->getSyncAppBase(),pev_itemsent,getDSConfig(),fItemsSent,getNumberOfChanges(),0);
+      DB_PROGRESS_EVENT(this,pev_itemsent,fItemsSent,getNumberOfChanges(),0);
     }
   }; // while not aborted and not message full
   // we are not done until all aNextMessageCommands are also out
@@ -871,8 +910,11 @@ localstatus TStdLogicDS::logicProcessMap(cAppCharP aRemoteID, cAppCharP aLocalID
 } // TStdLogicDS::logicProcessMap
 
 
+#endif // SYSYNC_SERVER
 
-#else
+
+
+#ifdef SYSYNC_CLIENT
 
 // Client Case
 // ===========
@@ -893,6 +935,10 @@ localstatus TStdLogicDS::startDataAccessForClient(void)
     &DBFuncTable, // context's function table
     this // datastore pointer needed for context
   );
+  // - datastoreinitscript might abort the sync with this datastore, check for that and exit if so
+  if (isAborted()) {
+    return getAbortStatusCode();
+  }
   #endif
   // - init post fetch filtering, sets fFilteringNeededForAll and fFilteringNeeded correctly
   initPostFetchFiltering();
@@ -902,25 +948,12 @@ localstatus TStdLogicDS::startDataAccessForClient(void)
 } // TStdLogicDS::startDataAccessForClient
 
 
-/// @brief called to have all non-yet-generated sync commands as "to-be-resumed"
-void TStdLogicDS::logicMarkOnlyUngeneratedForResume(void)
-{
-  // we do not maintain the map/bookmark list at this level, so
-  // derived class (ODBC, BinFile etc.) must make sure that their list is
-  // clean (no marks from previous sessions) before calling this inherited version
-  implMarkOnlyUngeneratedForResume();
-  // in client case, fItems does not contain ungenerated/unprocessed items
-  // so we don't have anything more do here for now
-} // TStdLogicDS::logicMarkOnlyUngeneratedForResume
-
-
-
 // called to generate sync sub-commands as client for remote server
 // @return true if now finished for this datastore
 bool TStdLogicDS::logicGenerateSyncCommandsAsClient(
   TSmlCommandPContainer &aNextMessageCommands,
   TSmlCommand * &aInterruptedCommandP,
-  const char *aLocalIDPrefix
+  cAppCharP aLocalIDPrefix
 )
 {
   localstatus sta = LOCERR_OK;
@@ -944,37 +977,52 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsClient(
       return false; // not complete
     }
     // read successful, test for EoC (end of changes)
-    if (fEoC)
-      break; // reading done
+    if (fEoC) break; // reading done
+    if (fSlowSync) changed=true; // all have changed (just in case GetItem does not return clean result here)
     // get sync op to perform
     TSyncOperation syncop=syncitemP->getSyncOp();
-    #ifdef OBJECT_FILTERING
-    // Filtering
-    // - call this anyway (makes sure item is made conformant to remoteAccept filter, even if
-    //   fFilteringNeeded is not set)
     if (syncop!=sop_delete && syncop!=sop_soft_delete && syncop!=sop_archive_delete) {
+      #ifdef OBJECT_FILTERING
+      // Filtering
+      // - call this anyway (makes sure item is made conformant to remoteAccept filter, even if
+      //   fFilteringNeeded is not set)
       bool passes=postFetchFiltering(syncitemP);
-      if (fFilteringNeeded) {
-      	if (!passes) {
-        	// item does not pass (current) filter: don't send it.
-          // Note that we DO NOT DELETE items falling out of the sync set by filtering,
-          // as for that we'd need to be able to differentiate adds from replaces.
-          // The use case for client-side filtering is also normally not the "moving-subset-window"
-          // case as for server side filtering, but more static exclusion of certain types of
-          // local entries (e.g. to prevent private stuff going to the server).
-          // - we don't need that sync item
-          delete syncitemP;
-          // - try next
-          continue;
+      if (fFilteringNeeded && !passes) {
+        // item was changed and does not pass now -> might be fallen out of the sync set now.
+        // Only when the DB is capable of tracking items fallen out of the sync set (i.e. bring them up as adds
+        // later should they match the filter criteria again), we can implement removing based
+        // on filter criteria. Otherwise, these are simply ignored.
+        if (implTracksSyncopChanges() && !fSlowSync && changed && (syncop==sop_replace || syncop==sop_wants_replace)) {
+          // item already exists on remote but falls out of syncset now: delete
+          // NOTE: This works only if reviewReadItem() is correctly implemented
+          //       and checks for items that are deleted after being reported
+          //       as replace to delete their local map entry (which makes
+          //       them add candidates again)
+          syncop = sop_delete;
+          syncitemP->cleardata(); // also get rid of unneeded data
+        }
+        else
+          syncop = sop_none; // ignore all others (especially adds or slowsync replaces)
+      }
+      else
+      #endif
+      {
+        // item passes (or no filters anyway) -> belongs to sync set
+        if ((syncop==sop_replace || syncop==sop_wants_replace) && !changed && !fSlowSync) {
+          // exists but has not changed since last sync
+          syncop=sop_none; // ignore for now
         }
       }
     }
-    #endif
-    // add local ID prefix, if any
-    if (aLocalIDPrefix && *aLocalIDPrefix && syncitemP->hasLocalID())
-      syncitemP->fLocalID.insert(0,aLocalIDPrefix);
+    // check if we should use that item
+    if (syncop==sop_none) {
+      delete syncitemP;
+      continue; // try next from DB
+    }
+    // set final syncop now
+    syncitemP->setSyncOp(syncop);
     // create sync op command
-    TSyncOpCommand *syncopcmdP = newSyncOpCommand(syncitemP,itemtypeP);
+    TSyncOpCommand *syncopcmdP = newSyncOpCommand(syncitemP,itemtypeP,aLocalIDPrefix);
     #ifdef CLIENT_USES_SERVER_DB
     // save item, we need it later for post-processing and Map simulation
     fItems.push_back(syncitemP);
@@ -989,7 +1037,7 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsClient(
     //   would be sent before the third..nth chunk of the previous command).
     TSmlCommand *cmdP = syncopcmdP;
     syncopcmdP=NULL;
-    // eventually, we have a NULL command here (e.g. in case it could not be generated due to MaxObjSize restrictions)
+    // possibly, we have a NULL command here (e.g. in case it could not be generated due to MaxObjSize restrictions)
     if (cmdP) {
       if (!fSessionP->issuePtr(cmdP,aNextMessageCommands,aInterruptedCommandP)) {
         alldone=false; // issue failed (no room in message), not finished so far
@@ -999,13 +1047,13 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsClient(
       fItemsSent++;
       // send event and check for abort
       #ifdef PROGRESS_EVENTS
-      if (!fSessionP->getSyncAppBase()->NotifyProgressEvent(pev_itemsent,getDSConfig(),fItemsSent,getNumberOfChanges())) {
+      if (!DB_PROGRESS_EVENT(this,pev_itemsent,fItemsSent,getNumberOfChanges(),0)) {
         implEndDataRead(); // terminate reading
         fSessionP->AbortSession(500,true,LOCERR_USERABORT);
         return false; // error
       }
       // check for "soft" suspension
-      if (!fSessionP->getSyncAppBase()->NotifyProgressEvent(pev_suspendcheck)) {
+      if (!SESSION_PROGRESS_EVENT(fSessionP,pev_suspendcheck,NULL,0,0,0)) {
         fSessionP->SuspendSession(LOCERR_USERSUSPEND);
       }
       #endif
@@ -1022,7 +1070,40 @@ bool TStdLogicDS::logicGenerateSyncCommandsAsClient(
 } // TStdLogicDS::logicGenerateSyncCommandsAsClient
 
 
-#endif // client case
+#endif // SYSYNC_CLIENT
+
+
+/// @brief called to have all not-yet-generated sync commands as "to-be-resumed"
+void TStdLogicDS::logicMarkOnlyUngeneratedForResume(void)
+{
+  if (IS_SERVER) {
+    #ifdef SYSYNC_SERVER
+    // we do not maintain the map/bookmark list at this level, so
+    // derived class (ODBC, BinFile etc.) must make sure that their list is
+    // clean (no marks from previous sessions) before calling this inherited version
+    implMarkOnlyUngeneratedForResume();
+    // Now add those that we have already received from the implementation
+    TSyncItemPContainer::iterator pos;
+    for (pos = fItems.begin(); pos != fItems.end(); ++pos) {
+      // let datastore mark these unprocessed
+      TSyncItem *syncitemP = (*pos);
+      // mark it for resume by ID
+      logicMarkItemForResume(syncitemP->getLocalID(),syncitemP->getRemoteID(),true); // these are unsent
+    }
+    #endif // SYSYNC_SERVER
+  }
+  else {
+    #ifdef SYSYNC_CLIENT
+    // we do not maintain the map/bookmark list at this level, so
+    // derived class (ODBC, BinFile etc.) must make sure that their list is
+    // clean (no marks from previous sessions) before calling this inherited version
+    implMarkOnlyUngeneratedForResume();
+    // in client case, fItems does not contain ungenerated/unprocessed items
+    // so we don't have anything more do here for now
+    #endif // SYSYNC_CLIENT
+  }
+} // TStdLogicDS::logicMarkOnlyUngeneratedForResume
+
 
 
 // called to process incoming item operation
@@ -1046,7 +1127,7 @@ bool TStdLogicDS::logicProcessRemoteItem(
     // start writing if not already started
     sta=startDataWrite();
     if (sta!=LOCERR_OK) {
-    	aStatusCommand.setStatusCode(sta);
+      aStatusCommand.setStatusCode(sta);
     }
     else {
       // show
@@ -1089,15 +1170,16 @@ bool TStdLogicDS::logicProcessRemoteItem(
           // - create new empty TMultiFieldItem
           TMultiFieldItem *refitemP =
             (TMultiFieldItem *) newItemForRemote(ity_multifield);
-          #ifdef SYSYNC_CLIENT
-          // Client: retrieve by local ID
-          refitemP->clearRemoteID(); // not known
-          refitemP->setLocalID(syncitemP->getLocalID()); // make sure we retrieve by local ID
-          #else
-          // Server: retrieve by remote ID
-          refitemP->clearLocalID(); // make sure we retrieve by remote ID
-          refitemP->setRemoteID(syncitemP->getRemoteID()); // make sure we retrieve by remote ID
-          #endif
+          if (IS_CLIENT) {
+            // Client: retrieve by local ID
+            refitemP->clearRemoteID(); // not known
+            refitemP->setLocalID(syncitemP->getLocalID()); // make sure we retrieve by local ID
+          }
+          else {
+            // Server: retrieve by remote ID
+            refitemP->clearLocalID(); // make sure we retrieve by remote ID
+            refitemP->setRemoteID(syncitemP->getRemoteID()); // make sure we retrieve by remote ID
+          }
           refitemP->setSyncOp(sop_replace);
           #ifdef OBJECT_FILTERING
           PDEBUGPRINTFX(DBG_DATA,(
@@ -1168,14 +1250,11 @@ bool TStdLogicDS::logicProcessRemoteItem(
             if (sta==404 || sta==410) {
               // this is an irregularity for a client (but it's perfectly
               // normal case for server, as client may use replace for adds+replaces)
-              #ifdef SYSYNC_CLIENT
-              if (!syncitemP->hasRemoteID()) {
+              if (IS_CLIENT && !syncitemP->hasRemoteID()) {
                 // - we cannot handle this properly, we have no remoteID, so report error to server
                 PDEBUGPRINTFX(DBG_ERROR,("TStdLogicDS: Item not found, but cannot switch to add because no RemoteID is known, Status=%hd",aStatusCommand.getStatusCode()));
               }
-              else
-              #endif
-              {
+              else {
                 if (fPreventAdd) {
                   // prevent implicit add -> return sta as is
                 }
@@ -1183,14 +1262,16 @@ bool TStdLogicDS::logicProcessRemoteItem(
                   // - switch to add if remote sent remoteID along so we can properly map
                   syncitemP->setSyncOp(sop_add);
                   PDEBUGPRINTFX(DBG_DATA,("TStdLogicDS: RetrieveItem: not found (Status=%hd) --> adding instead",sta));
-                  #ifndef SYSYNC_CLIENT
-                  // - make sure we delete the old item from the map table (as we *know* the item is gone - should it reappear under a different localID, it'll be re-added)
-                  implProcessMap(syncitemP->getRemoteID(),NULL);
-                  #endif
-                  // we can handle it, as we know the remoteID
-                  #ifdef SYSYNC_CLIENT
-                  irregular=true; // irregular only for client
-                  #endif
+                  if (IS_SERVER) {
+                    #ifdef SYSYNC_SERVER
+                    // server: make sure we delete the old item from the map table (as we *know* the item is gone - should it reappear under a different localID, it'll be re-added)
+                    implProcessMap(syncitemP->getRemoteID(),NULL);
+                    #endif
+                  }
+                  else {
+                    // client: we can handle it, as we know the remoteID
+                    irregular=true; // irregular only for client
+                  }
                   sta=LOCERR_OK; // ok for further processing, anyway
                 }
               }
@@ -1230,19 +1311,29 @@ bool TStdLogicDS::logicProcessRemoteItem(
           case sop_add :
             if (sta==418) {
               // 418: item already exists, this is kind of a conflict
-              // (should not happen normally, but can happen if aborted session was not
-              // completely rolled back by server, so treat it like
-              // "conflict resolved by client data winning")
-              PDEBUGPRINTFX(DBG_DATA,("to-be-added item already exists, and incomplete rollbacks in server possible -> trying replace (=conflict resolved by client winning)"));
-              // - switch to replace
-              syncitemP->setSyncOp(sop_replace);
-              irregular=true;
-              // - process again
-              if (implProcessItem(syncitemP,aStatusCommand)) {
-                aStatusCommand.setStatusCode(208); // client has won
+              if (isResuming() || fIgnoreUpdate || IS_CLIENT) {
+                // - in a client, this should not happen (prevented via checking against pending maps before
+                //   this routine is called) - if it still does just report the status back
+                // - in a resume, this can happen and the add should be ignored (se we return the status 418)
+                // - if updates are to be ignored, don't try update instead (and report 418)
+                // --> just return the status as-is
               }
               else {
-                sta = aStatusCommand.getStatusCode();
+                // in normal sync in the server case, this can happen when a previous session
+                // was aborted (and already applied adds not rolled back)
+                // --> reprocess it as a replace
+                PDEBUGPRINTFX(DBG_DATA,("to-be-added item already exists -> trying replace (=conflict resolved by client winning)"));
+                // - switch to replace
+                syncitemP->setSyncOp(sop_replace);
+                irregular=true;
+                // - process again
+                if (implProcessItem(syncitemP,aStatusCommand)) {
+                  aStatusCommand.setStatusCode(208); // client has won
+                }
+                else {
+                  // failed, return status
+                  sta = aStatusCommand.getStatusCode();
+                }
               }
             }
             break;
@@ -1250,28 +1341,27 @@ bool TStdLogicDS::logicProcessRemoteItem(
             if (sta==404 || sta==410) {
               // this is an irregularity for a client (but it's perfectly
               // normal case for server, as client may use replace for adds+replaces)
-              #ifdef SYSYNC_CLIENT
-              if (!syncitemP->hasRemoteID()) {
+              if (IS_CLIENT && !syncitemP->hasRemoteID()) {
                 // - we cannot handle this properly, we have no remoteID, so report error to server
                 PDEBUGPRINTFX(DBG_ERROR,("to-be-replaced item not found, but cannot switch to add because no RemoteID is known, Status=%hd",sta));
               }
-              else
-              #endif
-              {
+              else {
                 if (fPreventAdd) {
                   // prevent implicit add -> return status as is
                 }
                 else {
                   // - switch to add if remote sent remoteID along so we can properly map
                   syncitemP->setSyncOp(sop_add);
-                  #ifndef SYSYNC_CLIENT
-                  // - make sure we delete the old item from the map table (as we *know* the item is gone - should it reappear under a different localID, it'll be re-added)
-                  implProcessMap(syncitemP->getRemoteID(),NULL);
-                  #endif
-                  // we can handle it, as we know the remoteID
-                  #ifdef SYSYNC_CLIENT
-                  irregular=true;
-                  #endif
+                  if (IS_SERVER) {
+                    #ifdef SYSYNC_SERVER
+                    // - make sure we delete the old item from the map table (as we *know* the item is gone - should it reappear under a different localID, it'll be re-added)
+                    implProcessMap(syncitemP->getRemoteID(),NULL);
+                    #endif
+                  }
+                  else {
+                    // we can handle it, as we know the remoteID
+                    irregular=true;
+                  }
                   PDEBUGPRINTFX(DBG_DATA,("to-be-replaced item not found (Status=%hd) --> adding instead",sta));
                   // - process again (note that we are re-using the status command that might
                   //   already have a text item with an OS errir if something failed before)
@@ -1358,18 +1448,23 @@ localstatus TStdLogicDS::dsBeforeStateChange(TLocalEngineDSState aOldState,TLoca
 
   if (aNewState==dssta_dataaccessstarted) {
     // start data access
-    #ifdef SYSYNC_CLIENT
-    sta = startDataAccessForClient();
-    #else
-    sta = startDataAccessForServer();
-    #endif
+    if (IS_CLIENT) {
+      #ifdef SYSYNC_CLIENT
+      sta = startDataAccessForClient();
+      #endif
+    }
+    else {
+      #ifdef SYSYNC_SERVER
+      sta = startDataAccessForServer();
+      #endif
+    }
   }
-  #ifdef SYSYNC_CLIENT
-  if (aNewState==dssta_syncgendone) {
-    // when client has done sync gen, start writing
-    sta = startDataWrite();
-  }
-  #endif
+  if (IS_CLIENT) {
+    if (aNewState==dssta_syncgendone) {
+      // when client has done sync gen, start writing
+      sta = startDataWrite();
+    }
+  } // client
   if (aNewState==dssta_completed && !isAborted()) {
     // finish writing data now anyway
     endDataWrite();
