@@ -44,6 +44,10 @@ class CalDAVSource : public WebDAVSource,
     virtual std::string removeSubItem(const string &uid, const std::string &subid);
     virtual void flushItem(const string &uid);
     virtual std::string getSubDescription(const string &uid, const string &subid);
+    virtual void updateSynthesisInfo(SynthesisInfo &info,
+                                     XMLConfigFragments &fragments) {
+        info.m_backendRule = "HAVE-SYNCEVOLUTION-EXDATE-DETACHED";
+    }
 
     // implementation of SyncSourceLogging callback
     virtual std::string getDescription(const string &luid);
